libflag = false
local cp = console_print
if type(lib) == "table" and lib[0] == "LME" then
	libflag = true
	cp = lib.log_error
end

cp("Hitmarker:")

local crit_opac = gkini.ReadInt("hitmarker", "crit", 100) / 100
local hit_opac = gkini.ReadInt("hitmarker", "hit", 30) / 100
local hm_size = tostring(gkini.ReadInt("hitmarker", "size", 48))
local hm_handle = gkini.ReadString("hitmarker", "handle", "YES")

cp("	hit: " .. (hit_opac * 100) .. "%")
cp("	crit: " .. (crit_opac * 100) .. "%")
cp("	size: " .. hm_size)
cp("	default behavior: " .. hm_handle)

local hmimg = IMAGE_DIR .. "hit_indicator.png"
if not gksys.IsExist(hmimg) then
	cp("	Skin directory doesn't have a hit_indicator.png file; using default image instead")
	hmimg = "plugins/hitmarker/hit_indicator.png"
else
	cp("	Skin directory has hit_indicator.png and will be applied!")
end

local altimg = IMAGE_DIR .. "crit_indicator.png"
if not gksys.IsExist(altimg) then
	cp("	Skin directory doesn't have a crit_indicator.png file; will use magnification behavior instead")
	altimg = false
end

local function get_data()
	return {
		hit = hmimg,
		crit = altimg,
		o_hit = hit_opac,
		o_crit = crit_opac,
		size = hm_size,
		def_behavior = hm_handle,
	}
end

local hmihandle = iup.label {
	title = "",
	image = hmimg,
	alpha = "0",
	size = hm_size .. "x" .. hm_size,
	visible = "NO",
}

local function set_img(img_pointer, crit_pointer)
	if img_pointer and type(img_pointer) ~= "string" then
		img_pointer = "plugins/hitmarker/hit_indicator.png"
	end
	
	if crit_pointer and type(crit_pointer) ~= "string" then
		crit_pointer = false
	end
	
	if img_pointer and gksys.IsExist(img_pointer) then
		cp("hitmarker: image set by external as " .. img_pointer)
		hmihandle.image = img_pointer
		hmimg = img_pointer
		iup.Refresh(hmihandle)
	end
	
	if crit_pointer and gksys.IsExist(crit_pointer) then
		cp("hitmarker: crit set by external as " .. crit_pointer)
		altimg = crit_pointer
	end
end

local function set_opac(hit, crit)
	if hit and type(hit) ~= "number" then
		hit = 30
	end
	if crit and type(crit) ~= "number" then
		crit = 100
	end
	
	hit_opac = tonumber(hit or 30) / 100
	crit_opac = tonumber(crit or 100) / 100
end

local function set_size(num)
	num = tonumber(num)
	if num < 8 then
		num = 8
	end
	cp("hitmarker: size set by external as " .. tostring(num))
	hm_size = tostring(num)
	hmihandle.size = hm_size .. "x" .. hm_size
	iup.Refresh(hmihandle)
end

local function stop_behavior()
	cp("hitmarker: will not follow default behavior b external rule!")
	hm_handle = "NO"
end

local critflag = false
local function do_hit(crit_bool)
	--FadeControl(iup ihandle to affect, float seconds of transition, float start_alpha, float end_alpha)
	
	local htime = 0.20
	local hintensity = hit_opac
	
	if critflag == true then
		critflag = false
		hmihandle.image = hmimg
		hmihandle.size = tostring(hm_size) .. "x" .. tostring(hm_size)
		iup.Refresh(hmihandle)
	end
	
	if crit_bool == true then
		htime = 0.35
		hintensity = crit_opac
		critflag = true
		if altimg then
			hmihandle.image = altimg
		else
			hmihandle.size = tostring(hm_size * 1.5) .. "x" .. tostring(hm_size * 1.5)
		end
	end
	hmihandle.alpha = tostring(hintensity)
	hmihandle.visible = "YES"
	
	FadeControl(hmihandle, htime, hintensity, 0)
end

RegisterEvent(
	function()
		if hm_handle == "YES" then
			do_hit()
		end
	end,
	"PLAYER_HIT"
)

RegisterEvent(
	function(_, _, killer)
		if hm_handle == "YES" and killer == GetCharacterID() then
			do_hit(true)
		end
	end,
	"PLAYER_DIED"
)

local hitmarker_frame
local function create_hitmarker()
	if hitmarker_frame then
		if iup.IsValid(hmihandle) then
			do_hit(true)
			return
		end
	end
	
	if HUD and HUD.pluginlayer then
		
		local frame = iup.vbox {
			iup.fill { },
			iup.hbox {
				iup.fill { },
				hmihandle,
				iup.fill { },
			},
			iup.fill { },
		}
		
		if IsConnected() then
			hitmarker_frame = frame
			iup.Append(HUD.pluginlayer, hitmarker_frame)
			cp("hitmarker: applied to HUD")
		else
			iup.Detach(hmihandle)
			cp("hitmarker: player not in station, delaying application!")
			--hud is destroyed on login, which causes severe errors since we don't re-create
		end
	else
		cp("hitmarker: error applying to HUD! Is DefaultUI not loaded?")
	end
end

RegisterEvent(create_hitmarker, "HUD_SHOW")
create_hitmarker()

if libflag then
	lib.set_class("lx_hitmarker", "1.0.0", {
		set_opac = set_opac,
		set_size = set_size,
		set_img = set_img,
		do_hit = do_hit,
		stop_behavior = stop_behavior,
		get_data = get_data,
	})
	lib.lock_class("lx_hitmarker", "1.0.0")
end

RegisterUserCommand("hitmarker",
	function(_, args)
		if type(args) == "table" then
			if args[1] == "opac" then
				set_opac(tonumber(args[2] or 30), tonumber(args[3] or 100))
				print("Set hit opacity to " .. (args[2] or 30))
				print("Set crit opacity to " .. (args[3] or 100))
			elseif args[1] == "size" then
				set_size(tonumber(args[2] or 48))
				print("Set size to " .. (args[2] or 48))
			else
				print("Hitmarker plugin v1.0.0 Luxen De'Mark")
				print("usage:")
				print("/hitmarker opac # #")
				print("	set hit and crit opacity")
				print("/hitmarker size #")
				print("	set hitmarker size")
			end
		else
			print("Hitmarker plugin v1.0.0 Luxen De'Mark")
			print("usage:")
			print("/hitmarker opac # #")
			print("	set hit and crit opacity")
			print("/hitmarker size #")
			print("	set hitmarker size")			
		end
	end
)